package com.jz.dm.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.jz.common.constant.ResultMsg;
import com.jz.common.utils.Result;
import com.jz.dm.mapper.ApiInterfaceFileMapper;
import com.jz.dm.mapper.ApiOrgMapper;
import com.jz.dm.models.domian.ApiInterfaceFile;
import com.jz.dm.models.domian.ApiOrg;
import com.jz.dm.models.req.organizationManage.OrganizationManageAddReq;
import com.jz.dm.models.req.organizationManage.OrganizationManageDetailQueryReq;
import com.jz.dm.models.req.organizationManage.OrganizationManageListQueryReq;
import com.jz.dm.service.OrganizationManageService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Date;

/**
 * @author ZC
 * @PACKAGE_NAME: com.jz.dm.service.impl
 * @PROJECT_NAME: jz-dm-parent
 * @NAME: OrganizationManageImpl
 * @DATE: 2020-12-23/16:50
 * @DAY_NAME_SHORT: 周三
 * @Description:
 **/
@Service("organizationManageService")
@Slf4j
public class OrganizationManageImpl implements OrganizationManageService {

    @Resource
    private ApiOrgMapper apiOrgMapper;
    @Resource
    private ApiInterfaceFileMapper apiInterfaceFileMapper;

    /**
     * 组织列表信息查询
     *
     * @param req
     * @return
     */
    @Override
    public IPage<ApiOrg> listOrganization(OrganizationManageListQueryReq req) {
        IPage<ApiOrg> page = new Page<>(req.getPageNum(), req.getPageSize());
        QueryWrapper<ApiOrg> query = new QueryWrapper<>();

        if (StringUtils.isNotBlank(req.getOrgName())) {
            query.like("org_name", req.getOrgName());
        }
        if (StringUtils.isNotBlank(req.getLinkman())) {
            query.like("linkman", req.getLinkman());
        }
        if (StringUtils.isNotBlank(req.getOrgCode())) {
            query.eq("org_code", req.getOrgCode());
        }
        query.eq("is_deleted", 0);
        query.orderByDesc("create_date");
        return apiOrgMapper.selectPage(page, query);
    }

    /**
     * 获取组织详情
     *
     * @param req
     * @return
     */
    @Override
    public Result getOrganizationDetail(OrganizationManageDetailQueryReq req) {
        QueryWrapper<ApiOrg> query = new QueryWrapper<>();
        query.eq("is_deleted", 0);
        query.eq("id", req.getId());
        /*query.ne("status",0);*/
        return Result.of_success(apiOrgMapper.selectOne(query));
    }

    /**
     * 添加组织/更新
     *
     * @param req
     * @return
     */
    @Override
    public Result add(OrganizationManageAddReq req) {
        //更新组织
        if (null != req.getId()) {
            ApiOrg org = apiOrgMapper.selectById(req.getId());
            if (null == org) {
                return Result.of_error(ResultMsg.DATA_NOT_EXIST);
            }
            if (StringUtils.isNotBlank(req.getOrgName()) && !req.getOrgName().equals(org.getOrgName())) {
                if (checkOrgName(req.getOrgName())) {
                    return Result.of_error("组织名称已存在!");
                }
            }
            if (null != req.getOrgFolderId() && !req.getOrgFolderId().equals(org.getOrgFolderId())) {
                if (checkOrgFolderIsExist(req.getOrgFolderId())) {
                    return Result.of_error("目标文件夹不存在!");
                }
            }
            org.setUpdateDate(new Date());
            org.setOrgName(req.getOrgName());
            org.setLinkman(req.getLinkman());
            org.setOrgPhone(req.getOrgPhone());
            org.setOrgFolderId(req.getOrgFolderId());
            if (apiOrgMapper.updateById(org) > 0) {
                return Result.of_success(ResultMsg.UPDATE_SUCCESS);
            }
            //添加组织
        } else {
            ApiOrg apiOrg = new ApiOrg();
            BeanUtils.copyProperties(req, apiOrg);
            Result result = addOrg(req, apiOrg);
            if (result != null) return result;
        }
        return Result.of_error(ResultMsg.FAILURE);
    }

    /**
     * 校验组织名称是否存在
     *
     * @param newName
     * @return
     */
    private boolean checkOrgName(String newName) {
        if (StringUtils.isBlank(newName)) {
            return true;
        }
        QueryWrapper<ApiOrg> query = new QueryWrapper<>();
        query.eq("org_name", newName);
        query.eq("status", 1);
        query.eq("is_deleted", 0);
        ApiOrg apiOrg = apiOrgMapper.selectOne(query);
        if (null != apiOrg) {
            return true;
        }
        return false;
    }

    /**
     * 校验组织文件夹是否存在
     *
     * @param folderId
     * @return
     */
    private boolean checkOrgFolderIsExist(Long folderId) {
        if (null == folderId) {
            return true;
        }
        QueryWrapper<ApiInterfaceFile> query = new QueryWrapper<>();
        query.eq("id", folderId);
        query.eq("file_source", 2);
        query.eq("is_deleted", 0);
        ApiInterfaceFile apiInterfaceFile = apiInterfaceFileMapper.selectOne(query);
        if (null != apiInterfaceFile) {
            return true;
        }
        return false;
    }

    /**
     * 添加组织
     *
     * @param req
     * @param apiOrg
     * @return
     */
    private Result addOrg(OrganizationManageAddReq req, ApiOrg apiOrg) {
        if (StringUtils.isNotBlank(req.getOrgName())) {
            if (checkOrgName(req.getOrgName())) {
                return Result.of_error("组织名称已存在!");
            }
        }
        //生成组织编码
        String orgCode = generateCode(6);
        if (StringUtils.isBlank(orgCode)) {
            return Result.of_error("生成组织编码异常!");
        }
        apiOrg.setOrgCode(orgCode);//组织编码
        if (apiOrgMapper.insert(apiOrg) > 0) {
            return Result.of_success(ResultMsg.INSERT_SUCCESS);
        }
        return null;
    }

    /**
     * 生成组织编码
     * @param num  生成位数
     * @return
     */
    private String generateCode(int num) {
        QueryWrapper<ApiOrg> queryWra = new QueryWrapper<>();
        queryWra.select("org_code");
        queryWra.last("where id IN (SELECT MAX(id) FROM t_api_org)");
        ApiOrg apiOrg = apiOrgMapper.selectOne(queryWra);
        String orgCode = null;
        if (null != apiOrg) {
            Integer maxOrgCode = Integer.valueOf(apiOrg.getOrgCode());
            orgCode = String.format("%0" + num + "d",maxOrgCode+1);
        }
        return orgCode;
    }

    /**
     * 注销组织
     *
     * @param id
     * @return
     */
    @Override
    public Result logoutOrg(Long id) {
        ApiOrg apiOrg = apiOrgMapper.selectById(id);
        if (null == apiOrg) {
            return Result.of_error(ResultMsg.DATA_NOT_EXIST);
        }
        UpdateWrapper<ApiOrg> delete = new UpdateWrapper<>();
        delete.set("is_deleted", 1);
        delete.set("status", 0);
        delete.eq("id", id);
        if (apiOrgMapper.update(null, delete) > 0) {
            return Result.of_success(ResultMsg.UPDATE_SUCCESS);
        }
        return Result.of_success(ResultMsg.UPDATE_FAIL);
    }

    /**
     * 获取认证组织信息
     *
     * @param apiOrgId
     * @return
     */
    @Override
    public ApiOrg getAuthOrganization(Long apiOrgId) {
        QueryWrapper<ApiOrg> query = new QueryWrapper<>();
        query.eq("id", apiOrgId);
        query.eq("status", 1);
        query.eq("is_deleted", 0);
        return apiOrgMapper.selectOne(query);
    }

}
